if (select(2, UnitClass("player")) ~= "DEATHKNIGHT") then
  SnowfallRune = nil;
  return;
end


local min = math.min;
local max = math.max;
local floor = math.floor;
local huge = math.huge;


--------------------------------------------------------------------------------
-- Texture status bar class

local textureStatusBar = {};
setmetatable(textureStatusBar, {__index = getmetatable(UIParent).__index});

local textureLow = 5 / 64;
local textureMid = 32 / 64;
local textureHigh = 1 - textureLow;
local maxTextureOffset = textureMid - textureLow;

-- Contructor
function textureStatusBar:new(frameName, parent)
  local frame = CreateFrame("Frame", frameName, parent);
  getmetatable(frame).__index = textureStatusBar;

  if (parent) then
    frame:SetFrameLevel(parent:GetFrameLevel() + 1);
  end
  frame:SetFrameStrata("LOW");

  frame.bar = CreateFrame("Frame", nil, frame);
  frame.bar:SetFrameLevel(frame:GetFrameLevel() + 1);
  frame.bar:SetFrameStrata("LOW");
  frame.bar:SetPoint("LEFT");
  frame.bar:SetPoint("RIGHT");
  frame.bar:SetPoint("TOP");
  frame.bar:SetPoint("BOTTOM");

  frame.foundation = frame.bar:CreateTexture(nil, "BACKGROUND");
  frame.foundation:SetTexture(0, 0, 0, 1);
  frame.foundation:SetAllPoints();

  frame.textures = {};
  for i = 1, 4 do
    frame.textures[i] = frame.bar:CreateTexture(nil, "BORDER");
    frame.textures[i]:SetAlpha(0.25);
    frame.textures[i]:SetBlendMode("ADD");
  end
  for i = 1, 4 do
    frame.textures[i]:SetAllPoints();
    if ((i % 2) == 0) then
      frame.textures[i].left = textureLow;
      frame.textures[i].right = textureMid;
    else
      frame.textures[i].left = textureMid;
      frame.textures[i].right = textureHigh;
    end
    if (floor(i / 2) == 0) then
      frame.textures[i].top = textureLow;
      frame.textures[i].bottom = textureMid;
    else
      frame.textures[i].top = textureMid;
      frame.textures[i].bottom = textureHigh;
    end
  end

  frame.high = 1;
  frame.low = 0;
  frame.anchor = "LOW";
  frame.textureLength = 1;

  return frame;
end

function textureStatusBar:setBackgroundTexture(...)
  if (not self.background) then
    self.background = self:CreateTexture();
    self.background:SetAllPoints();
  end

  self.background:SetTexture(...);
end

function textureStatusBar:setBrightness(value)
  local b = 0.25 * value;
  for i = 1, 4 do
    self.textures[i]:SetAlpha(b);
  end
end

function textureStatusBar:setTexture(...)
  for i = 1, 4 do
    self.textures[i]:SetTexture(...);
  end
end

function textureStatusBar:setTextureAnchor(anchor)
  if (anchor ~= "HIGH" and anchor ~= "LOW") then
    error([[textureStatusbar:setTextureAnchor, anchor must be one of "HIGH" or "LOW"]]);
  end
  self.anchor = anchor;
end

function textureStatusBar:setTextureLength(value)
  self.textureLength = value;
end

function textureStatusBar:setHigh(value)
  self.high = value;
end

function textureStatusBar:setLow(value)
  self.low = value;
end

function textureStatusBar:update()
  local height = self:GetHeight();
  self.bar:SetPoint("BOTTOM", self, "BOTTOM", 0, height * max(0, min(1, self.low)));
  self.bar:SetPoint("TOP", self, "BOTTOM", 0, height * max(0, min(1, self.high)));

  local topOffset, bottomOffset;
  if (self.anchor == "LOW") then
    topOffset = max(0, min(1, (1 - (min(1, self.high) - self.low) / self.textureLength))) * maxTextureOffset;
    bottomOffset = max(0, min(1, (0 - self.low) / self.textureLength)) * maxTextureOffset;
  else
    topOffset = max(0, min(1, (self.high - 1) / self.textureLength)) * maxTextureOffset;
    bottomOffset = max(0, min(1, (1 - (self.high - max(0, self.low)) / self.textureLength))) * maxTextureOffset;
  end
  for _, texture in ipairs(self.textures) do
    texture:SetTexCoord(texture.left, texture.right, texture.top + topOffset, texture.bottom - bottomOffset);
  end
end


--------------------------------------------------------------------------------
-- Settings

-- There are currently two groups of settings: default and install.
-- The default group is shipped as Addons/SnowfallRune/SnowfallRuneSettings/SnowfallRuneSettings.lua.
-- The install group is user-specified in Addons/SnowfallRuneSettings/SnowfallRuneSettings.lua.
-- Fields in the install group override fields the default group.
SnowfallRune.config.default = SnowfallRune.settings;
SnowfallRune.settings = nil;
if (SnowfallRune.config.install) then
  setmetatable(SnowfallRune.config.install, {__index = SnowfallRune.config.default});
else
  SnowfallRune.config.install = SnowfallRune.config.default;
end
local c = SnowfallRune.config.install;

local SPELLID_ICY_TOUCH = 49909;
local SPELLID_PLAGUE_STRIKE = 49921;
local SPELLID_PESTILENCE = 50842;
local SPELLID_FROST_FEVER = 55095;
local SPELLID_BLOOD_PLAGUE = 55078;

local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

local runeColorsFoundationDim = {
  [RUNETYPE_BLOOD]  = {1.0 * 0.4, 0.0 * 0.4, 0.0 * 0.4},
  [RUNETYPE_UNHOLY] = {0.0 * 0.4, 0.5 * 0.4, 0.0 * 0.4},
  [RUNETYPE_FROST]  = {0.0 * 0.4, 1.0 * 0.4, 1.0 * 0.4},
  [RUNETYPE_DEATH]  = {0.8 * 0.4, 0.1 * 0.4, 1.0 * 0.4},
};

local runeColorsFoundationBright = {
  [RUNETYPE_BLOOD]  = {1.0 * 0.6, 0.0 * 0.6, 0.0 * 0.6},
  [RUNETYPE_UNHOLY] = {0.0 * 0.6, 0.5 * 0.6, 0.0 * 0.6},
  [RUNETYPE_FROST]  = {0.0 * 0.6, 1.0 * 0.6, 1.0 * 0.6},
  [RUNETYPE_DEATH]  = {0.8 * 0.6, 0.1 * 0.6, 1.0 * 0.6},
};

local runeTextures = {
  [RUNETYPE_BLOOD]  = "Interface\\Icons\\Spell_DeathKnight_BloodPresence",
  [RUNETYPE_UNHOLY] = "Interface\\Icons\\Spell_DeathKnight_UnholyPresence",
  [RUNETYPE_FROST]  = "Interface\\Icons\\Spell_DeathKnight_FrostPresence",
  [RUNETYPE_DEATH]  = "Interface\\Icons\\Spell_DeathKnight_ArmyOfTheDead",
};

local nextGcdDuration = 0;
local gcdEnd = 0;

local frostFeverDuration = 0;
local frostFeverEnd = 0;
local bloodPlagueDuration = 0;
local bloodPlagueEnd = 0;

local baseDiseaseDuration = 15;
local diseaseDurationPerEpidemicPoint = 3;
local diseaseDuration = baseDiseaseDuration;

local pestilenceTargetGUID = nil;
local pestilenceTargetFrostFeverEnd = 0;
local pestilenceTargetBloodPlagueEnd = 0;
local pestilenceOtherDiseaseEnd = 0;

local runeFrame;
local runeFrameTexture;
local gcd;
local gcdTexture;
local runeBars = {};
local frostFeverLongBar, frostFeverShortBar;
local bloodPlagueLongBar, bloodPlagueShortBar;
local pestilenceLongBar, pestilenceShortBar;

local GUIDToUnitId;

local runeFirstReadys = {};
for i = 1, 6 do
  runeFirstReadys[i] = -huge;
end



--------------------------------------------------------------------------------
-- OnUpdate

local function OnUpdate()
  local time = GetTime();

  -- Update rune bars
  local runeBar;
  local start, duration, runeReady;
  local cooldownEnd;
  for rune = 1, 6 do
    runeBar = runeBars[rune];
    start, duration, runeReady = GetRuneCooldown(rune);

    if (runeReady) then
      runeBar.longBar.high = 1;
      runeBar.shortBar.high = runeBar.shortBar.low + 2.5 / 2.5;
    else
      -- Remember when the rune will first be ready (not counting misses)
      cooldownEnd = start + duration;
      if (cooldownEnd > time + 1.5) then
        runeFirstReadys[rune] = cooldownEnd;
      end

      runeBar.longBar.high = max(0, min(1, (time - start) / duration));
      runeBar.shortBar.high = (time - (start + duration - 2.5)) / 2.5;
    end
    runeBar.shortBar.low = (time - runeFirstReadys[rune]) / 2.5;

    runeBar.longBar:update();
    runeBar.shortBar:update()
  end

  -- Update GCD
  gcd:SetPoint("TOP", 0, -c.runeBarPadding - c.runeBarHeight * min(max(gcdEnd - time, 0), 2.5) / 2.5);
  gcd:SetPoint("BOTTOM", runeFrame, "TOP", 0, -c.runeBarPadding - c.runeBarHeight * min(max(gcdEnd - time + nextGcdDuration, nextGcdDuration), 2.5) / 2.5);

  if (c.showDiseases) then
    -- Update frost fever
    frostFeverLongBar.low = 1 - (frostFeverEnd - time) / frostFeverDuration;
    frostFeverLongBar:update();
    frostFeverShortBar.low = 1 - (frostFeverEnd - time) / 2.5;
    frostFeverShortBar.high = frostFeverShortBar.low + 3 / 2.5;
    frostFeverShortBar:update();

    -- Update blood plague
    bloodPlagueLongBar.low = 1 - (bloodPlagueEnd - time) / bloodPlagueDuration;
    bloodPlagueLongBar:update();
    bloodPlagueShortBar.low = 1 - (bloodPlagueEnd - time) / 2.5;
    bloodPlagueShortBar.high = bloodPlagueShortBar.low + 3 / 2.5;
    bloodPlagueShortBar:update();
  end

  if (c.showPestilence) then
    -- Update pestilence
    local targetGUID = UnitGUID("target");
    local pestilenceEnd =
      targetGUID == pestilenceTargetGUID and pestilenceOtherDiseaseEnd
      or max(pestilenceTargetFrostFeverEnd, pestilenceTargetBloodPlagueEnd);
    pestilenceLongBar.low = 1 - (pestilenceEnd - time) / diseaseDuration;
    pestilenceLongBar:update();
    pestilenceShortBar.low = 1 - (pestilenceEnd - time) / 2.5;
    pestilenceShortBar.high = pestilenceShortBar.low + 3 / 2.5;
    pestilenceShortBar:update();
  end

  -- Update melee range indication
  if (c.showMeleeRange) then
    local spellNamePlagueStrike = GetSpellInfo(SPELLID_PLAGUE_STRIKE);
    if (IsSpellInRange(spellNamePlagueStrike, "target") == 0) then
      runeFrameTexture:SetTexture(0.4, 0.4, 0.4, 1);
    else
      runeFrameTexture:SetTexture(0, 0, 0, 1);
    end
  end
end



--------------------------------------------------------------------------------
-- Event handlers

local eventHandlers = {};

local function RuneTextureUpdate(rune)
  runeBar = runeBars[rune];
  if (not runeBar) then
    return;
  end
  local runeType = GetRuneType(rune);
  if (runeType) then
    runeBar.longBar:setTexture(runeTextures[runeType]);
    runeBar.shortBar:setTexture(runeTextures[runeType]);
    runeBar.longBar.foundation:SetTexture(unpack(runeColorsFoundationDim[runeType]));
    runeBar.shortBar.foundation:SetTexture(unpack(runeColorsFoundationBright[runeType]));
  end
end

function eventHandlers.PLAYER_ENTERING_WORLD()
  for rune in pairs(RuneFrame.runes) do
    RuneTextureUpdate(rune);
  end

  eventHandlers.UPDATE_SHAPESHIFT_FORM();
  eventHandlers.ACTIONBAR_UPDATE_COOLDOWN();
  eventHandlers.PLAYER_TALENT_UPDATE();
end

function eventHandlers.PLAYER_TALENT_UPDATE()
  local _, _, _, _, talentPoints = GetTalentInfo(3, 4);  -- Epidemic
  diseaseDuration = baseDiseaseDuration + talentPoints * diseaseDurationPerEpidemicPoint;
end
eventHandlers.PLAYER_ALIVE = eventHandlers.PLAYER_TALENT_UPDATE;

function eventHandlers.RUNE_TYPE_UPDATE(self, event, rune)
  -- Rune switching to or from a death rune
  RuneTextureUpdate(rune);
end

function eventHandlers.UPDATE_SHAPESHIFT_FORM()
  if (GetShapeshiftForm() == 3) then
    -- Unholy presence
    nextGcdDuration = 1.0;
  else
    -- Not unholy presence
    nextGcdDuration = 1.5;
  end
end

function eventHandlers.ACTIONBAR_UPDATE_COOLDOWN()
  local gcdStart, gcdDuration, gcdIsEnabled = GetCompanionCooldown("CRITTER", 1);
  -- GetCompanionCooldown is buggy and can sometimes end cooldowns early, returning gcdDuration=0. If this happens, we'll just keep our same gcdEnd time.
  gcdEnd = gcdIsEnabled > 0 and gcdDuration > 0 and gcdStart + gcdDuration or gcdEnd;
end

local function UpdateDiseases()
  local _;

  _, _, _, _, _, frostFeverDuration, frostFeverEnd = UnitAura("target", GetSpellInfo(SPELLID_FROST_FEVER), nil, "HARMFUL PLAYER");
  frostFeverDuration = frostFeverDuration or 0;
  frostFeverEnd = frostFeverEnd or 0;

  _, _, _, _, _, bloodPlagueDuration, bloodPlagueEnd = UnitAura("target", GetSpellInfo(SPELLID_BLOOD_PLAGUE), nil, "HARMFUL PLAYER");
  bloodPlagueDuration = bloodPlagueDuration or 0;
  bloodPlagueEnd = bloodPlagueEnd or 0;
end

function eventHandlers.COMBAT_LOG_EVENT_UNFILTERED(self, event, timestamp, subEvent, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId)
  if (subEvent == "SPELL_CAST_SUCCESS") then
    if ((spellId == SPELLID_ICY_TOUCH or spellId == SPELLID_PLAGUE_STRIKE or spellId == SPELLID_PESTILENCE) and destGUID ~= pestilenceTargetGUID and sourceGUID == UnitGUID("player")) then
      pestilenceTargetGUID = destGUID;
      pestilenceOtherDiseaseEnd, pestilenceTargetFrostFeverEnd, pestilenceTargetBloodPlagueEnd =
        max(pestilenceTargetFrostFeverEnd, pestilenceTargetBloodPlagueEnd), pestilenceOtherDiseaseEnd, pestilenceOtherDiseaseEnd;
    end
    -- todo, this is only necessary because SPELL_AURA_REFRESH is buggy
    if (spellId == SPELLID_PESTILENCE and sourceGUID == UnitGUID("player")) then
      local unitId = GUIDToUnitId(destGUID);
      if (unitId and (UnitAura(unitId, GetSpellInfo(SPELLID_FROST_FEVER), nil, "HARMFUL PLAYER") or UnitAura(unitId, GetSpellInfo(SPELLID_BLOOD_PLAGUE), nil, "HARMFUL PLAYER"))) then
        pestilenceOtherDiseaseEnd = GetTime() + diseaseDuration;
      end
    end
  elseif (subEvent == "SPELL_AURA_REMOVED") then
    if (destGUID == pestilenceTargetGUID and sourceGUID == UnitGUID("player")) then
      if (spellId == SPELLID_FROST_FEVER) then
        pestilenceTargetFrostFeverEnd = 0;
      elseif (spellId == SPELLID_BLOOD_PLAGUE) then
        pestilenceTargetBloodPlagueEnd = 0;
      end
    end
  elseif (subEvent == "UNIT_DIED") then
    -- Pestilence
    if (destGUID == pestilenceTargetGUID) then
      pestilenceTargetGUID = nil;
      pestilenceTargetFrostFeverEnd = 0;
      pestilenceTargetBloodPlagueEnd = 0;
    end

    -- Diseases
    if (destGUID == UnitGUID("target")) then
      UpdateDiseases();
    end
  end
end

function eventHandlers.UNIT_AURA(self, event, unitID)
  -- Diseases
  if (unitID == "target") then
    UpdateDiseases();
  end

  -- Pestilence
  -- todo, this is only necessary because SPELL_AURA_REFRESH is buggy
  local _, _, _, _, _, pestilenceFrostFeverDuration, pestilenceFrostFeverEnd = UnitAura(unitID, GetSpellInfo(SPELLID_FROST_FEVER), nil, "HARMFUL PLAYER");
  local _, _, _, _, _, pestilenceBloodPlagueDuration, pestilenceBloodPlagueEnd = UnitAura(unitID, GetSpellInfo(SPELLID_BLOOD_PLAGUE), nil, "HARMFUL PLAYER");
  pestilenceFrostFeverEnd = pestilenceFrostFeverEnd or 0;
  pestilenceBloodPlagueEnd = pestilenceBloodPlagueEnd or 0;
  if (UnitGUID(unitID) == pestilenceTargetGUID) then
    pestilenceTargetFrostFeverEnd = pestilenceFrostFeverEnd;
    pestilenceTargetBloodPlagueEnd = pestilenceBloodPlagueEnd;
  else
    pestilenceOtherDiseaseEnd = max(pestilenceOtherDiseaseEnd, pestilenceFrostFeverEnd, pestilenceBloodPlagueEnd);
  end
end

eventHandlers.PLAYER_TARGET_CHANGED = UpdateDiseases;



--------------------------------------------------------------------------------
-- Create widgets

-- Disable the default rune frame
RuneFrame:UnregisterAllEvents();
RuneFrame:Hide();

-- Create a new rune frame
runeFrame = CreateFrame("Frame", nil, UIParent);
runeFrame:SetFrameStrata("LOW");
runeFrame:SetFrameLevel(runeFrame:GetParent():GetFrameLevel() + 1);
runeFrame:SetPoint(unpack(c.runeFramePosition));
runeFrame:SetHeight(c.runeBarHeight + c.runeBarPadding * 2);
runeFrame:SetWidth(c.runeBarWidth * 6 + c.runeBarPadding * 7 + (c.showDiseases and 1 or 0) * (2 * c.diseaseBarWidth + c.diseaseBarPadding + c.runeBarPadding) + (c.showPestilence and 1 or 0) * (c.pestilenceBarWidth + c.diseaseBarPadding));
runeFrame:SetScale(c.runeFrameScale);

-- Create a black texture for the rune frame
runeFrameTexture = runeFrame:CreateTexture();
runeFrameTexture:SetTexture(0, 0, 0, 1);
runeFrameTexture:SetAllPoints(runeFrame);

-- Create a GCD frame
gcd = CreateFrame("Frame", nil, runeFrame);
gcd:SetFrameStrata("LOW");
gcd:SetFrameLevel(gcd:GetParent():GetFrameLevel() + 1);
gcd:SetPoint("LEFT", c.runeBarPadding, 0);
gcd:SetPoint("RIGHT", -c.runeBarPadding, 0);
gcd:SetPoint("TOP", 0, -c.runeBarPadding);
gcd:SetPoint("BOTTOM", runeFrame, "TOP", 0, -c.runeBarPadding - (runeFrame:GetHeight() - 2 * c.runeBarPadding) * 1.5 / 2.5);

-- Create a white texture for the GCD frame
gcdTexture = gcd:CreateTexture();
gcdTexture:SetTexture(1, 1, 1, 1);
gcdTexture:SetAllPoints();

local runeBar;
for i = 1, 6 do
  runeBars[i] = {};
  runeBar = runeBars[i];
  runeBarLeftToRightIndex = _G["RuneButtonIndividual"..i]:GetID();  -- Display runes in the same order that Blizzard does

  runeBar.longBar = textureStatusBar:new(nil, runeFrame);
  runeBar.longBar:SetFrameLevel(gcd:GetFrameLevel() + 1);
  runeBar.longBar:SetHeight(c.runeBarHeight);
  runeBar.longBar:SetWidth(c.runeBarWidth);
  runeBar.longBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + (runeBarLeftToRightIndex - 1) * (c.runeBarWidth + c.runeBarPadding), -c.runeBarPadding);
  runeBar.longBar:setBackgroundTexture(0.15, 0.15, 0.15, 1);
  runeBar.longBar:setBrightness(0.2);
  runeBar.longBar.anchor = "LOW";

  runeBar.shortBar = textureStatusBar:new(nil, runeFrame);
  runeBar.shortBar:SetFrameLevel(runeBar.longBar:GetFrameLevel() + 1);
  runeBar.shortBar:SetHeight(c.runeBarHeight);
  runeBar.shortBar:SetWidth(c.runeBarWidth);
  runeBar.shortBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + (runeBarLeftToRightIndex - 1) * (c.runeBarWidth + c.runeBarPadding), -c.runeBarPadding);
  runeBar.shortBar:setBrightness(0.4);
  runeBar.shortBar.anchor = "LOW";
  runeBar.shortBar.textureLength = 2.5 / 2.5;
end

if (c.showDiseases) then
  -- Create a long-duration frost fever bar
  frostFeverLongBar = textureStatusBar:new(nil, runeFrame);
  frostFeverLongBar:SetFrameLevel(gcd:GetFrameLevel() + 1);
  frostFeverLongBar:SetHeight(c.runeBarHeight);
  frostFeverLongBar:SetWidth(c.diseaseBarWidth);
  frostFeverLongBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding), -c.runeBarPadding);
  frostFeverLongBar:setBackgroundTexture(0.15, 0.15, 0.15, 1);
  frostFeverLongBar:setTexture("Interface\\Icons\\Spell_DeathKnight_FrostFever");
  frostFeverLongBar:setBrightness(0.6);
  frostFeverLongBar.anchor = "HIGH";

  -- Create a short-duration frost fever bar
  frostFeverShortBar = textureStatusBar:new(nil, runeFrame);
  frostFeverShortBar:SetFrameLevel(frostFeverLongBar:GetFrameLevel() + 1);
  frostFeverShortBar:SetHeight(c.runeBarHeight);
  frostFeverShortBar:SetWidth(c.diseaseBarWidth);
  frostFeverShortBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding), -c.runeBarPadding);
  frostFeverShortBar:setTexture("Interface\\Icons\\Spell_DeathKnight_FrostFever");
  frostFeverShortBar.anchor = "LOW";
  frostFeverShortBar.textureLength = 3 / 2.5;

  -- Create a long-duration blood plague bar
  bloodPlagueLongBar = textureStatusBar:new(nil, runeFrame);
  bloodPlagueLongBar:SetFrameLevel(gcd:GetFrameLevel() + 1);
  bloodPlagueLongBar:SetHeight(c.runeBarHeight);
  bloodPlagueLongBar:SetWidth(c.diseaseBarWidth);
  bloodPlagueLongBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding) + c.diseaseBarWidth + c.diseaseBarPadding, -c.runeBarPadding);
  bloodPlagueLongBar:setBackgroundTexture(0.15, 0.15, 0.15, 1);
  bloodPlagueLongBar:setTexture("Interface\\Icons\\Spell_DeathKnight_BloodPlague");
  bloodPlagueLongBar:setBrightness(0.6);
  bloodPlagueLongBar.anchor = "HIGH";

  -- Create a short-duration blood plague bar
  bloodPlagueShortBar = textureStatusBar:new(nil, runeFrame);
  bloodPlagueShortBar:SetFrameLevel(bloodPlagueLongBar:GetFrameLevel() + 1);
  bloodPlagueShortBar:SetHeight(c.runeBarHeight);
  bloodPlagueShortBar:SetWidth(c.diseaseBarWidth);
  bloodPlagueShortBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding) + c.diseaseBarWidth + c.diseaseBarPadding, -c.runeBarPadding);
  bloodPlagueShortBar:setTexture("Interface\\Icons\\Spell_DeathKnight_BloodPlague");
  bloodPlagueShortBar.anchor = "LOW";
  bloodPlagueShortBar.textureLength = 3 / 2.5;
end

if (c.showPestilence) then
  -- Create a long-duration pestilence bar
  pestilenceLongBar = textureStatusBar:new(nil, runeFrame);
  pestilenceLongBar:SetFrameLevel(gcd:GetFrameLevel() + 1);
  pestilenceLongBar:SetHeight(c.runeBarHeight);
  pestilenceLongBar:SetWidth(c.diseaseBarWidth);
  pestilenceLongBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding) + (c.showDiseases and 2 or 0) * (c.diseaseBarWidth + c.diseaseBarPadding), -c.runeBarPadding);
  pestilenceLongBar:setBackgroundTexture(0.15, 0.15, 0.15, 1);
  pestilenceLongBar:setTexture("Interface\\Icons\\Spell_Shadow_PlagueCloud");
  pestilenceLongBar:setBrightness(0.6);
  pestilenceLongBar.anchor = "HIGH";

  -- Create a short-duration pestilence bar
  pestilenceShortBar = textureStatusBar:new(nil, runeFrame);
  pestilenceShortBar:SetFrameLevel(pestilenceLongBar:GetFrameLevel() + 1);
  pestilenceShortBar:SetHeight(c.runeBarHeight);
  pestilenceShortBar:SetWidth(c.diseaseBarWidth);
  pestilenceShortBar:SetPoint("TOPLEFT", runeFrame, "TOPLEFT", c.runeBarPadding + 6 * (c.runeBarWidth + c.runeBarPadding) + (c.showDiseases and 2 or 0) * (c.diseaseBarWidth + c.diseaseBarPadding), -c.runeBarPadding);
  pestilenceShortBar:setTexture("Interface\\Icons\\Spell_Shadow_PlagueCloud");
  pestilenceShortBar.anchor = "LOW";
  pestilenceShortBar.textureLength = 3 / 2.5;
end

-- Enable event handlers
runeFrame:SetScript(
  "OnEvent",
  function(self, event, ...)
    eventHandlers[event](self, event, ...);
  end
);
for k, v in pairs(eventHandlers) do
  runeFrame:RegisterEvent(k);
end

-- Enable OnUpdate handler
runeFrame:SetScript("OnUpdate", OnUpdate);


do
  local unitIds = {
    "target",
    "focus",
    "mouseover",
    "pettarget",
    "arena1target",
    "arena2target",
    "arena3target",
    "arena4target",
    "arena5target",
    "party1target",
    "party2target",
    "party3target",
    "party4target",
    "raid1target",
    "raid2target",
    "raid3target",
    "raid4target",
    "raid5target",
    "raid6target",
    "raid7target",
    "raid8target",
    "raid9target",
    "raid10target",
    "raid11target",
    "raid12target",
    "raid13target",
    "raid14target",
    "raid15target",
    "raid16target",
    "raid17target",
    "raid18target",
    "raid19target",
    "raid20target",
    "raid21target",
    "raid22target",
    "raid23target",
    "raid24target",
    "raid25target",
    "raid26target",
    "raid27target",
    "raid28target",
    "raid29target",
    "raid30target",
    "raid31target",
    "raid32target",
    "raid33target",
    "raid34target",
    "raid35target",
    "raid36target",
    "raid37target",
    "raid38target",
    "raid39target",
    "raid40target",
  }
  function GUIDToUnitId(GUID)
    for k, v in ipairs(unitIds) do
      if (UnitGUID(v) == GUID) then
        return (v);
      end
    end
    return nil;
  end
end
